<?php
// AutoFilter.php
// CC GNU GPL（http://creativecommons.org/licenses/GPL/2.0/deed.ja）
// by Daisuke Nogami (http://null-null.net)
// at 2007/12/30

/////////////////////////////////////////////////////////////////////////
// MagpieRSSの設定
/////////////////////////////////////////////////////////////////////////

require_once("./magpierss/rss_fetch.inc");
define("MAGPIE_OUTPUT_ENCODING", "UTF-8");
define("MAGPIE_CACHE_ON", false);


/////////////////////////////////////////////////////////////////////////
// 日本語文字コードの設定
/////////////////////////////////////////////////////////////////////////

mb_language("ja");
mb_internal_encoding("UTF-8");


/////////////////////////////////////////////////////////////////////////
// AutoFilter（このプログラム）の設定
/////////////////////////////////////////////////////////////////////////

// 抽出したい記事に含まれていそうなキーワードの一覧（正規表現）
$filter = "";
// 投稿先のメールアドレス
$mailAddress = "";
// 記事のRSS（RSS0.9とRSS1.0に対応）
$rssPath = "";
// 投稿履歴を保存しておく為のファイル
$lastEntryDataPath = "lastentry.dat";


/////////////////////////////////////////////////////////////////////////
// main
/////////////////////////////////////////////////////////////////////////

$autoFilter = new AutoFilter($filter, $mailAddress, $rssPath, $lastEntryDataPath);
if($autoFilter->load()){
	$autoFilter->update();
	// 抽出したエントリの書き出し（デバグ用）
	print($autoFilter->write());
}


/////////////////////////////////////////////////////////////////////////
// AutoFilter
/////////////////////////////////////////////////////////////////////////

class AutoFilter {
	var $filter; // 抽出したい記事に含まれていそうなキーワードの一覧
	var $mailAddress; // 投稿先のメールアドレス
	var $rssPath; // 記事のRSS（RSS0.9とRSS1.0に対応）
	var $rss; // 読み込んだRSS
	var $lastEntryDataPath; // 投稿履歴を保存しておく為のファイル
	var $lastEntryData; // 投稿履歴
	var $entryArr; // エントリリスト
	
	function AutoFilter($filter, $mailAddress, $rssPath, $lastEntryDataPath){
		$this->filter = $filter;
		$this->mailAddress = $mailAddress;
		$this->rssPath = $rssPath;
		$this->lastEntryDataPath = $lastEntryDataPath;
	}
	
	// RSSと履歴ファイルの読み込み
	function load(){
		// RSSの読み込み
		$this->rss = $this->loadRSS($this->rssPath);
		// 履歴ファイルの読み込み
		$this->lastEntryData = $this->loadLastEntryData($this->lastEntryDataPath);
		return $this->rss ? ture : false;
	}
	
	// RSSの読み込み
	function loadRSS($path){
		$rss = fetch_rss($path);
		return $rss;
	}
	
	// 履歴ファイルの読み込み
	function loadLastEntryData($path){
		if($fp = fopen($path, 'r')){
			if(flock($fp, LOCK_SH)){
				while(!feof($fp)){
					$lastEntryData .= fgets($fp);
				}
			}
			flock($fp, LOCK_UN);
		}
		fclose($fp);
		return $lastEntryData;
	}
	
	// 新しいエントリが無いか調べて有ったら履歴ファイルを更新してポスト
	function update(){
		// 読み込んだRSSからエントリリストを作成
		$this->entryArr = $this->packRss($this->rss);

		// 最新のエントリの取得
		$newEntryArr = $this->newEntryArr($this->filter, $this->entryArr, $this->lastEntryData);
		$newEntryData = '';
		foreach($newEntryArr as $entry){
			$newEntryData .= $entry->link . "\n";
		}
		
		// 最新のエントリがある場合はそのエントリを投稿する
		if($newEntryData != ''){
			// 履歴ファイルの更新
			$this->updateLastEntryData($this->lastEntryDataPath, $newEntryData);
			// ついでにこっちも更新
			$this->lastEntryData .= $newEntryData;
			// エントリの投稿
			$this->postEntry($this->mailAddress, $newEntryArr);
			print("post!<br />");
		}
	}
	
	// RSSからエントリリストを作成
	function packRss($rss){
		$entryArr = array();
		foreach($rss->items as $item){
			$title = $item[title];
			$link = $item[link];
			$description = $item[content][encoded] == '' ? $item[description] : $item[content][encoded];
			$source = $item[ag][source] == '' ? $item[dc][publisher] : $item[ag][source];
			array_push($entryArr, new Entry($title, $link, $description, $source));
		}
		return $entryArr;
	}
	
	// 最新のエントリの抽出
	function newEntryArr($filter, $entryArr, $lastEntryData){
		$newEntryArr = array();
		// 新しいエントリを探す
		foreach($entryArr as $entry){
			// タイトル+本文とパタンを比較
			$content = $entry->title . ' ' . $entry->description;
			// 内容が求めるものと一致するか
			if(preg_match($filter, $content)){
				// 過去に投稿されていないか
				if(strpos($lastEntryData, $entry->link) === FALSE){
					array_push($newEntryArr, $entry);
				}
			}
		}
		return $newEntryArr;
	}
	
	// 履歴ファイルの更新
	function updateLastEntryData($path, $data){
		// 追加モードでオープン
		if($fp = fopen($path, 'a')){
			if(flock($fp, LOCK_SH)){
				fwrite($fp, $data);
			}
			flock($fp, LOCK_UN);
		}
		fclose($fp);
	}
	
	// エントリの投稿
	function postEntry($mailAddress, $entryArr){
		foreach($entryArr as $entry){
			$subject = $entry->title;
			$body = '<p>';
			$body .= 'via ';
			$body .= '<a href="' . $entry->link . '">';
			$body .= $entry->title;
			$body .= $entry->source == '' ? '' : ('@' . $entry->source); // sourceが無い場合がある
			$body .= '</a></p>';
			$body .= '<blockquote cite="' . $entry->link . '">' . $entry->description . '</blockquote>';
			mb_send_mail($mailAddress, $subject, $body);
		}
	}
	
	// 抽出したエントリの書き出し（デバグ用）
	function write(){
		$html = '';
		$cnt = 0;
		
		// page title
		$html .= '<h1 style="background-color:#000;color:#fff;font-size:140%;">';
		$html .= $this->filter . 'にマッチする記事一覧';
		$html .= '</h1>';
		
		foreach($this->entryArr as $entry){
			$content = $entry->title . ' ' . $entry->description;
			if(!preg_match($this->filter, $content)){
				continue;
			}
	
			// section title
			$html .= '<h2 style="font-size:130%;font-weight:bold;margin-top:30px;margin-bottom:0;background-color:#bbb">';
			$html .= $entry->title;
			$html .= '</h2>';
			
			// body
			$html .= '<p>';
			$html .= 'via ';
			$html .= '<a href="' . $entry->link . '">';
			$html .= $entry->title;
			$html .= $entry->source == '' ? '' : ('@' . $entry->source); // sourceが無い場合がある
			$html .= '</a></p>';
			$html .= '<blockquote cite="' . $entry->link . '" style="border-left:1px solid #000;padding:10px;">' . $entry->description . '</blockquote>';
		}
		
		return $html;
	}
}


/////////////////////////////////////////////////////////////////////////
// Entry
/////////////////////////////////////////////////////////////////////////

class Entry {
	var $title;
	var $link;
	var $description;
	var $source;
	
	// title, link, descriptionが必須
	// sourceはオプション
	function Entry($title, $link, $description, $source){
		$this->title = $title;
		$this->link = $link;
		$this->description = $description;
		$this->source = $source;
	}
}

?>